@@ -1,25 +1,29 @@
 =encoding utf8
-	
+
 =head1 NAME
 
 IRI - Internationalized Resource Identifiers
 
 =head1 VERSION
 
-This document describes IRI version 0.003_02
+This document describes IRI version 0.004
 
 =head1 SYNOPSIS
 
   use IRI;
   
-  my $i	= IRI->new(value => 'https://example.org:80/index#frag');
+  my $i = IRI->new(value => 'https://example.org:80/index#frag');
   say $i->scheme; # 'https'
   say $i->path; # '/index'
 
   my $base = IRI->new(value => "http://www.hestebedg\x{e5}rd.dk/");
-  my $i	= IRI->new(value => '#frag', base => $base);
+  my $i = IRI->new(value => '#frag', base => $base);
   say $i->abs; # 'http://www.hestebedgård.dk/#frag'
 
+  # Defer parsing of the IRI until necessary
+  my $i = IRI->new(value => "http://www.hestebedg\x{e5}rd.dk/", lazy => 1);
+  say $i->path; # path is parsed here
+
 =head1 DESCRIPTION
 
 The IRI module provides an object representation for Internationalized
@@ -27,6 +31,20 @@ Resource Identifiers (IRIs) as defined by
 L<RFC 3987|http://www.ietf.org/rfc/rfc3987.txt> and supports their parsing,
 serializing, and base resolution.
 
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< lazy >>
+
+A boolean value indicating whether the IRI should be parsed (and validated)
+during object construction (false), or parsed only when an IRI component is
+accessed (true). If no components are ever needed (e.g. an IRI is constructed
+with a C<< value >> and C<< value >> is the only accessor ever called), no
+parsing will take place.
+
+=back
+
 =head1 METHODS
 
 =over 4
@@ -61,16 +79,18 @@ Returns the respective component of the parsed IRI.
 package IRI {
 	use v5.14;
 	use warnings;
-	our $VERSION	= '0.003_02';
+	our $VERSION	= '0.004_01';
 	use Moo;
 	use MooX::HandlesVia;
-	use Types::Standard qw(Str InstanceOf HashRef);
+	use Types::Standard qw(Str InstanceOf HashRef Bool);
 	use Scalar::Util qw(blessed);
 	
 # 	class_type 'URI';
 # 	coerce 'IRI' => from 'Str' => via { IRI->new( value => $_ ) };
 # 	coerce 'IRI' => from 'URI' => via { IRI->new( value => $_->as_string ) };
 
+	has 'lazy' => (is => 'ro', isa => Bool, default => 0);
+	has '_initialized' => (is => 'rw', isa => Bool, default => 0, init_arg => undef);
 	has 'base' => (is => 'ro', isa => InstanceOf['IRI'], predicate => 'has_base', coerce => sub {
 		my $base	= shift;
 		if (blessed($base)) {
@@ -115,9 +135,19 @@ package IRI {
 	
 	sub BUILD {
 		my $self	= shift;
-		my $comp	= $self->_parse_components($self->value);
+		unless ($self->lazy) {
+			my $comp	= $self->_parse_components($self->value);
+		}
 	}
 	
+	before [qw(components as_string abs resolved_components)] => sub {
+		my $self	= shift;
+		if (not $self->_initialized) {
+# 			warn "Lazily initializing IRI";
+			my $comp	= $self->_parse_components($self->value);
+		}
+	};
+
 	# These regexes are (mostly) from the syntax grammar in RFC 3987
 	my $HEXDIG			= qr<[0-9A-F]>o;
 	my $ALPHA			= qr<[A-Za-z]>o;
@@ -211,7 +241,7 @@ package IRI {
 		my $v		= shift;
 		my $c;
 		
-		if ($v =~ /^${IRIreference}$/) {
+		if ($v =~ /^${IRIreference}$/o) {
 			%$c = %+;
 		} else {
 			use Data::Dumper;
@@ -220,6 +250,7 @@ package IRI {
 		
 		$c->{path}	//= '';
 		$self->_set_components($c);
+		$self->_initialized(1);
 	}
 	
 	sub _merge {
